<?php
/*
Plugin Name: Woo Customer & Order Notes
Description: Adds Ordernote (Admin Only), Ordernote (Show to Customer), and Customernote fields to WooCommerce orders for admin/shopadmin use only.
Version: 1.4
Author: EasyToolsWP
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// Hook to add custom meta box
add_action( 'add_meta_boxes', 'woo_add_custom_notes_metabox' );

// Add custom meta box to the order edit page
function woo_add_custom_notes_metabox() {
    add_meta_box(
        'woo_custom_notes_metabox',  // ID of the meta box
        __( 'Customer Notes', 'woocommerce' ),  // Title
        'woo_display_custom_notes_metabox',  // Callback function
        'shop_order',  // Post type
        'side',  // Context (side column)
        'default'  // Priority
    );
}

// Display the custom meta box fields (Admin Ordernote, Customer Ordernote, and Customernote)
function woo_display_custom_notes_metabox( $post ) {
    $order = wc_get_order( $post->ID );
    
    // Retrieve the notes
    $ordernote_admin = get_post_meta( $post->ID, '_ordernote_admin', true );
    $ordernote_customer = get_post_meta( $post->ID, '_ordernote_customer', true );
    $customer_email = $order->get_billing_email();
    $customernote = get_option( 'customernote_' . $customer_email, '' ); // Retrieve note tied to email

    if ( current_user_can( 'manage_woocommerce' ) ) {
        // Ordernote (Admin Only)
        echo '<p><strong>' . __( 'Ordernote (Admin only)', 'woocommerce' ) . '</strong></p>';
        echo '<textarea id="ordernote_admin" name="ordernote_admin" style="width:100%; height: 75px;">' . esc_attr( $ordernote_admin ) . '</textarea>';

        // Ordernote (Show to Customer)
        echo '<p><strong>' . __( 'Ordernote (Show to Customer)', 'woocommerce' ) . '</strong></p>';
        echo '<textarea id="ordernote_customer" name="ordernote_customer" style="width:100%; height: 75px;">' . esc_attr( $ordernote_customer ) . '</textarea>';

        // Customernote tied to email address
        echo '<p><strong>' . __( 'Customernote (Admin only)</strong><br>', 'woocommerce' ) . esc_html( $customer_email ) . '</p>';
        echo '<textarea id="customernote" name="customernote" style="width:100%; height: 75px;">' . esc_attr( $customernote ) . '</textarea>';
    }
}

// Save the Ordernotes and Customernote when order is updated
add_action( 'woocommerce_process_shop_order_meta', 'woo_save_custom_notes', 10, 2 );

function woo_save_custom_notes( $order_id, $post ) {
    if ( current_user_can( 'manage_woocommerce' ) ) {
        // Save Ordernote (Admin Only)
        if ( isset( $_POST['ordernote_admin'] ) ) {
            update_post_meta( $order_id, '_ordernote_admin', sanitize_text_field( $_POST['ordernote_admin'] ) );
        }

        // Save Ordernote (Show to Customer)
        if ( isset( $_POST['ordernote_customer'] ) ) {
            update_post_meta( $order_id, '_ordernote_customer', sanitize_text_field( $_POST['ordernote_customer'] ) );
        }

        // Save Customernote based on email
        $order = wc_get_order( $order_id );
        $customer_email = $order->get_billing_email();

        if ( isset( $_POST['customernote'] ) && ! empty( $customer_email ) ) {
            update_option( 'customernote_' . $customer_email, sanitize_text_field( $_POST['customernote'] ) );
        }
    }
}

// Display Ordernote (Show to Customer) on the customer emails and order page
add_action( 'woocommerce_email_order_details', 'woo_show_ordernote_to_customer', 20, 4 );
add_action( 'woocommerce_order_details_after_order_table', 'woo_show_ordernote_to_customer_in_account', 10, 1 );

function woo_show_ordernote_to_customer( $order, $sent_to_admin, $plain_text, $email ) {
    $ordernote_customer = get_post_meta( $order->get_id(), '_ordernote_customer', true );
    if ( ! empty( $ordernote_customer ) ) {
        echo '<p><strong>' . __( 'Note:', 'woocommerce' ) . '</strong> ' . esc_html( $ordernote_customer ) . '</p>';
    }
}

function woo_show_ordernote_to_customer_in_account( $order ) {
    $ordernote_customer = get_post_meta( $order->get_id(), '_ordernote_customer', true );
    if ( ! empty( $ordernote_customer ) ) {
        echo '<p><strong>' . __( 'Note:', 'woocommerce' ) . '</strong> ' . esc_html( $ordernote_customer ) . '</p>';
    }
}

// Add custom columns to the orders list (optional: displaying notes)
add_filter( 'manage_edit-shop_order_columns', 'woo_add_custom_columns' );
function woo_add_custom_columns( $columns ) {
    $new_columns = array();
    foreach ( $columns as $key => $column ) {
        $new_columns[ $key ] = $column;
        if ( 'order_status' === $key ) { // Add after the status column
            $new_columns['ordernote_admin'] = __( 'Ordernote (Admin)', 'woocommerce' );
            $new_columns['ordernote_customer'] = __( 'Ordernote (Customer)', 'woocommerce' );
        }
    }
    return $new_columns;
}

// Display data in the custom columns
add_action( 'manage_shop_order_posts_custom_column', 'woo_display_custom_column_data', 10, 2 );
function woo_display_custom_column_data( $column, $post_id ) {
    if ( 'ordernote_admin' === $column ) {
        $ordernote_admin = get_post_meta( $post_id, '_ordernote_admin', true );
        echo esc_html( $ordernote_admin );
    }
    if ( 'ordernote_customer' === $column ) {
        $ordernote_customer = get_post_meta( $post_id, '_ordernote_customer', true );
        echo esc_html( $ordernote_customer );
    }
}
